//  Created by Frank M. Carrano and Tim Henry.
//  Copyright (c) 2013 __Pearson Education__. All rights reserved.
//  Modified by J Braun to test SortedList with Events

#include <iostream>
#include <string>
#include "LinkedList.h" // ADT list operations
#include "LinkedSortedList.h" // ADT list operations
#include "ListQueue.h" // ADT Queue operations
#include "Event.h"

using namespace std;

void displayList(ListInterface<string>* listPtr)
{
	cout << "The list contains " << endl;
   for (int pos = 1; pos <= listPtr->getLength(); pos++)
   {
      cout << listPtr->getEntry(pos) << " ";
   } // end for
	cout << endl << endl;
}  // end displayList

void listTester()
{
	ListInterface<string>* listPtr = new LinkedList<string>();
	cout << "Testing the Link-Based List:" << endl;
	string data[] = {"one", "two", "three", "four", "five", "six"};
   cout << "isEmpty: returns " << listPtr->isEmpty() << "; should be 1 (true)" << endl;
	for (int i = 0; i < 6; i++)
   {
		if (listPtr->insert(i + 1, data[i]))
         cout << "Inserted " << listPtr->getEntry(i + 1)
         << " at position " << (i + 1) << endl;
      else
         cout << "Cannot insert " << data[i] << " at position " << (i + 1)
         << endl;
	}  // end for
   
   displayList(listPtr);
   
   cout << "isEmpty: returns " << listPtr->isEmpty() << "; should be 0 (false)" << endl;
   cout << "getLength returns : " << listPtr->getLength() << "; should be 6" << endl;
   
   cout << "The entry at position 4 is " << listPtr->getEntry(4) << "; should be four" << endl;
   cout << "After replacing the entry at position 3 with XXX: ";
   listPtr->setEntry(3, "XXX");
   displayList(listPtr);
   
   try
   {
      cout << "Attempt an illegal retrieval from position 6: " << endl;
      listPtr->getEntry(6);
   }
   catch(PrecondViolatedExcep e)
   {
      cout << e.what() << endl;
   }  // end try/catch
} // end listTester

void displayList(SortedListInterface<Event>* listPtr)
{
	cout << "The sorted list contains " << endl;
   for (int pos = 1; pos <= listPtr->getLength(); pos++)
   {
      cout << listPtr->getEntry(pos);
   } // end for
	cout << endl << endl;
}  // end displayList

void listEventTester(SortedListInterface<Event>* listPtr)
{
   listPtr->insertSorted(Event('A',8,5));
   listPtr->insertSorted(Event('A',6,7));
   listPtr->insertSorted(Event('A',16,5));
   listPtr->insertSorted(Event('D',12,0));
   listPtr->insertSorted(Event('A',12,4));
   listPtr->insertSorted(Event('D',16,4));
   displayList(listPtr);
   
   cout << "isEmpty: returns " << listPtr->isEmpty() << "; should be 0 (false)" << endl;
   cout << "getLength returns : " << listPtr->getLength() << "; should be 6" << endl;
   
   cout << "remove(2): returns " << listPtr->remove(2) << "; should be 1 (true)" << endl;
   cout << "remove(1): returns " << listPtr->remove(1) << "; should be 1 (true)" << endl;
   cout << "remove(6): returns " << listPtr->remove(6) << "; should be 0 (false)" << endl;
   displayList(listPtr);
   cout << "getLength returns : " << listPtr->getLength() << "; should be 4" << endl;
   cout << "clear: " << endl;
   listPtr->clear();
   cout << "isEmpty: returns " << listPtr->isEmpty() << "; should be 1 (true)" << endl;
   
   try
   {
      cout << "Attempt an illegal retrieval from position 6: " << endl;
      listPtr->getEntry(6);
   }
   catch(PrecondViolatedExcep e)
   {
      cout << e.what() << endl;
   }
}

void queueTester()
{
	QueueInterface<string>* queuePtr = new ListQueue<string>();

	string items[] = {"one", "two", "three", "four", "five", "six"};
	cout << "Empty: " << queuePtr->isEmpty() << endl;
	for (int i = 0; i < 6; i++)
   {
		cout<<"Adding " << items[i] << endl;
      bool success = queuePtr->enqueue(items[i]);
      if (!success)
         cout << "Failed to add " << items[i] << " to the queue." << endl;
	}  // end for
   
	cout << "Empty?: " << queuePtr->isEmpty() << endl;
	
	for (int i = 0; i < 6; i++)
   {
		cout << "Loop " << i << endl;
      
      try
      {
         cout << "peekFront: " << queuePtr->peekFront() << endl;
      }
      catch (PrecondViolatedExcep e)
      {
         cout << e.what() << endl;
      }  // end try/catch
      
		cout << "Empty: " << queuePtr->isEmpty() << endl;
		cout << "dequeue: " << queuePtr->dequeue() << endl;
	}  // end for
}  // end queueTester

int main()
{

 //Test the linked list implementation
  listTester();

 //Test the queue
  cout << endl << "Testing the List-Based Queue:" << endl;
  cout << "======================================" << endl;
  queueTester();

 //Test the sorted list implementation
  SortedListInterface<Event>* eListPtr = new LinkedSortedList<Event>();
  cout << endl << "Testing the Link-Based Sorted List:" << endl;
  cout << "======================================" << endl;
  listEventTester(eListPtr);

   return 0;
}  // end main

