/** @file Event.h
 *  Example of what the Event class might contain
 *  Might need to also overload the equality operator
 */

#ifndef _EVENT_H
#define _EVENT_H
#include <ostream>

class Event
{
  friend std::ostream &operator<<(std::ostream &, const Event &);
public:
  Event(char t='D', int tm=0, int tt=0):type(t), time(tm), ttime(tt){}
  char getType() const;
  int getTime() const;
  int getTransTime() const;
  void set(char t, int tm, int tt);

 /** Overload < operator so events can be sorted by time,
  *  and then by type (A or D) if times are equal.)
  */
  bool operator>(const Event& rhs)const; 
  bool operator<(const Event& rhs)const; 
  bool operator<=(const Event& rhs)const; 
  bool operator==(const Event& rhs)const; 
  bool operator>=(const Event& rhs)const; 
  bool operator!=(const Event& rhs)const; 

private:
  char type;	//D is departure, A is arrival
  int time;	//arrival or departure time
  int ttime;	//Transaction time for Arrival events
};

#endif

